import React, {Component} from 'react';
import ReactDOM from 'react-dom';
import {Easer} from 'functional-easing';
import {Timeline} from 'react-imation/timeline';
import {rotate, percent} from 'react-imation/tween-value-factories';
import Scrubber from './Scrubber';
import {Motion} from 'react-motion';

const easeOutBounce = new Easer().using('out-bounce');
const easeOutSine = new Easer().using('out-sine');

const MIN_TIME = 0;
const MAX_TIME = 100;

const styles = {
  scrubber: {
    width: '100%',
    boxSizing: 'border-box',
  },
  ball: {
    borderRadius: '50%',
    width: '20px',
    height: '20px',
    position: 'absolute',
    backgroundColor: 'white',
    opacity: 0.2
  }
};

class App extends Component {

  render() {
    return (
      <Timeline
        playOnMount={true}
        min={MIN_TIME}
        max={MAX_TIME}
        loop={true}>
      {({tween,time, playing, togglePlay, setTime}) => {

        const top = (100 + 40 * Math.sin(time/5));
        const left = tween([
          [MIN_TIME, 0],
          [MAX_TIME, 100]
        ], easeOutSine);

        return <div style={{padding: 30}}>

          <Motion style={{top,left}}>
          {interpolated =>
            <div
              style={{
                ...styles.ball,
                top: interpolated.top,
                left: interpolated.left + '%',
              }} />
          }</Motion>

          <div
            style={{
              ...styles.ball,
              top,
              left: left + '%'
            }}/>

          <h2
            style={tween([
              [MIN_TIME, { transform: rotate(0) }],
              [MAX_TIME, { transform: rotate(360) }]
            ], easeOutBounce)}>
            spin
          </h2>

          <button onClick={togglePlay}>
            {playing ? 'pause' : 'play'}
          </button>

          <Scrubber
            style={styles.scrubber}
            min={MIN_TIME}
            max={MAX_TIME}
            value={time}
            onChangeValue={setTime} />

        </div>}

      }</Timeline>
    )
  }
}

ReactDOM.render(<App/>, document.getElementById('example'));
